@@ -26,6 +26,7 @@ import os.path
 import collections
 import operator
 import functools
+import inspect
 
 saved_dispatch = pickle.Pickler.dispatch.copy()
 saved_dispatch_table = pickle.dispatch_table.copy()
@@ -139,7 +140,8 @@ _preservedModules = {}
 def restore_modules_entry(doDel, old, new, preserveReferenceToNew=True):
     """Restore the content of sys.modules."""
     try:
-        _preservedModules[id(new)] = new
+        if preserveReferenceToNew:
+            _preservedModules[id(new)] = new
         if doDel and sys.modules.has_key(new.__name__) and old == ():
             del sys.modules[new.__name__]
         if old != ():
@@ -391,13 +393,14 @@ class Pickler(pickle.Pickler):
         if isinstance(obj, types.ModuleType):
             if self.saveModule(obj):
                 return
-            
+
+        super_save = self.__class__.__bases__[0].save
         if isinstance(obj, types.FrameType):
             trace_func = obj.f_trace
             if trace_func is not None:
                 obj.f_trace = self.TraceFunctionSurrogate()
             try:
-                return self.super_save(obj)
+                return super_save(self, obj)
             finally:
                 if trace_func is not None:
                     obj.f_trace = trace_func
@@ -430,6 +433,11 @@ class Pickler(pickle.Pickler):
             getattr(obj,"__name__", None) in ('__new__', '__subclasshook__')):
             return self.saveBuiltinNew(obj)
 
+        if ( isinstance(obj, types.BuiltinMethodType) and 
+             hasattr(obj, "__self__") and 
+             obj.__self__ is not None ):
+            return self.saveBuiltinMethod(obj)
+
         # avoid problems with some classes, that implement 
         # __getattribute__ or __getattr__ in a way, 
         # that getattr(obj, "__setstate__", None) raises an exception.
@@ -438,11 +446,7 @@ class Pickler(pickle.Pickler):
         except Exception, e:
             raise UnpicklingWillFailError("Object %r has hostile attribute access: %r" % (obj, e))
                 
-        self.super_save(obj)
- 
-    def super_save(self, obj):
-        return self.__class__.__bases__[0].save(self, obj)
-    
+        return super_save(self, obj)
     def memoize(self, obj):
         """Store an object in the memo."""
         try:
@@ -588,6 +592,7 @@ class Pickler(pickle.Pickler):
         return self.save_reduce(obj=obj, *rv)
 
     def saveClass(self, obj):
+        write = self.write
         f = type(obj)
         name = obj.__name__
         d1 = {}; d2 = {}
@@ -600,9 +605,20 @@ class Pickler(pickle.Pickler):
             if k in ('__dict__', '__class__' ):
                 continue
             d2[k] = v
-        self.save_reduce(f,(name, obj.__bases__, d1), (None,d2), obj=obj)
-    
-    
+        self.save_reduce(f,(name, obj.__bases__, d1), obj=obj)
+        
+        # Use setattr to set the remaining class members. 
+        # unfortunately, we can't use the state parameter of the 
+        # save_reduce method (pickle BUILD-opcode), because BUILD 
+        # invokes a __setstate__ method eventually inherited from 
+        # a super class.
+        keys = d2.keys()
+        keys.sort()
+        for k in keys:
+            v = d2[k]
+            self.save_reduce(setattr, (obj, k, v))
+            write(pickle.POP)
+
     def saveModule(self, obj, reload=False):
         write = self.write
         
@@ -632,7 +648,7 @@ class Pickler(pickle.Pickler):
             # create the module
             self.save_reduce(create_module, (type(obj), obj.__name__, getattr(obj, "__doc__", None)), obj.__dict__, obj=obj)
         finally:
-            restore_modules_entry(True, savedModule, obj)
+            restore_modules_entry(True, savedModule, obj, preserveReferenceToNew=False)
             
         if doDel or not reload:
             write(pickle.TUPLE3+pickle.REDUCE)
@@ -679,6 +695,14 @@ class Pickler(pickle.Pickler):
             raise pickle.PicklingError("Can't pickle %r: it's not the same object as %s.%s" %
                         (obj, t, obj.__name__))
         return self.save_reduce(getattr, (t, obj.__name__), obj=obj)
+    
+    def saveBuiltinMethod(self, obj):
+        objSelf = obj.__self__
+        objName = obj.__name__
+        if getattr(objSelf, objName, None) != obj:
+            raise pickle.PicklingError("Can't pickle %r: it's not the same object as %s.%s" %
+                        (obj, objSelf, objName))
+        return self.save_reduce(getattr, (objSelf, objName), obj=obj)
         
     def saveWrapperOrMethodDescriptor(self, obj):
         t = obj.__objclass__
@@ -775,11 +799,12 @@ class SPickleTools(object):
         return cls.loads(str, persistent_load)
     
     @classmethod
-    def loads(cls, str, persistent_load=None):
+    def loads(cls, str, persistent_load=None, useCPickle=True):
         if str.startswith("BZh9"):
             str = decompress(str)
         file = StringIO(str)
-        unpickler = cPickle.Unpickler(file)
+        p = cPickle if useCPickle else pickle
+        unpickler = p.Unpickler(file)
         if persistent_load is not None:
             unpickler.persistent_load = persistent_load
         return unpickler.load()
@@ -836,3 +861,65 @@ class SPickleTools(object):
             pickle = self.dumps_with_external_ids(r, idmap, matchNetref=True)
             return (pickle, idmap)
         return returnWrapper
+
+    @classmethod
+    def module_for_globals(cls, callable_or_moduledict, withDefiningModules=False):
+        """Get the module associated with a callable or a module dictionary.
+
+        If you pickle a module, make sure to keep a reference to unpickled module.
+        Otherwise the __del__-method of the module will clear the modules 
+        dictionary. Usually, the sPickle code for serializing modules, preserves 
+        a reference to modules created from a pickle but not imported into 
+        sys.modules. However, there might be cases, where you need to identify 
+        relevant modules yourself. This method can be used, to find the relevant module(s).
+
+        Return value: None, or a module or a set of modules
+        """
+        g = None
+        modules = []
+        result = set()
+        if isinstance(callable_or_moduledict, dict):
+            g = callable_or_moduledict
+        else:
+            func = callable_or_moduledict
+            if withDefiningModules:
+                try:
+                    modules.append(func.__module__)
+                except Exception:
+                    pass
+            if inspect.ismethod(func):
+                func = func.im_func
+                if withDefiningModules:
+                    try:
+                        modules.append(func.__module__)
+                    except Exception:
+                        pass
+            if inspect.isfunction(func):
+                g = func.func_globals
+        for i, v in enumerate(modules):
+            if v:
+                m = sys.modules.get(v)
+                if m is not None:
+                    result.add(m)
+        if g is not None:
+            for m in sys.modules.itervalues():
+                if g is getattr(m, '__dict__', None):
+                    result.add(m)
+        if len(result) == 0:
+            return None
+        if len(result) == 1:
+            return result.pop()
+        return result
+
+
+class StacklessTaskletReturnValueException(BaseException):
+    """This exception can be used to return a value from a Stackless Python tasklet.
+    
+    Usually a tasklet has no way to return a value at the end of its live. But it can 
+    raise an exception. Therefore we use this exception to encapsulate a return value.
+    
+    Because this exception is not a real error and we do not want this exception 
+    to be caught, we derive it from BaseException instead of Exception.
+    """
+    def __init__(self, value):
+        self.value = value
